'''
  Copyright (C) 2013 by Christopher Cooper, Lorena Barba

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
'''
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt
import matplotlib.gridspec as gridspec
import os
import numpy
import sys
import math
sys.path.append('../util')
from an_solution import *

def scanOutput(filename):
    
    flag = 0
    for line in file(filename):
        line = line.split()
        if len(line)>0:
            if line[0]=='Converged':
                iterations = int(line[2])
            if line[0]=='Total' and line[1]=='elements':
                N = int(line[-1])
            if line[0]=='Totals:':
                flag = 1
            if line[0]=='Esolv' and flag==1:
                Esolv = float(line[2])
            if line[0]=='Esurf' and flag==1:
                Esurf = float(line[2])
            if line[0]=='Ecoul' and flag==1:
                Ecoul = float(line[2])
            if line[0]=='Time' and flag==1:
                Time = float(line[2])

    return N, iterations, Esolv, Esurf, Ecoul, Time
            

mesh = numpy.array([1,2,4,8])

comm = './main.py PyGBeOrientation_1IGTconvergence/inputs/1IGT.param PyGBeOrientation_1IGTconvergence/inputs/1IGT_sensor_d0'
out = 'output_aux'

print 'Runs for molecule + set phi/dphi surface'
Nel = numpy.zeros(len(mesh))
iterations = numpy.zeros(len(mesh))
Esolv = numpy.zeros(len(mesh))
Esurf = numpy.zeros(len(mesh))
Ecoul = numpy.zeros(len(mesh))
Time = numpy.zeros(len(mesh))
for i in range(len(mesh)):
    print 'Start run for mesh '+str(mesh[i])
    cmd = comm + str(mesh[i]) + '_031-130.config > ' + out
    os.system(cmd)
    print 'Scan output file'
    Nel[i],iterations[i],Esolv[i],Esurf[i],Ecoul[i],Time[i] = scanOutput(out)
    print 'Esolv: %f, Esurf: %f'%(Esolv[i], Esurf[i])

total_time = Time

Etot = Esolv+Esurf

Etot_extra = (Etot[-1]*Etot[-3]-Etot[-2]*Etot[-2])/(Etot[-1]-2*Etot[-2]+Etot[-3])

Etot_error = abs(Etot-Etot_extra)/abs(Etot_extra)

print '\nNumber of elements : '+str(Nel)
print 'Number of iteration: '+str(iterations)
print 'Etotal extrapolated: %f kcal/mol'%Etot_extra
print 'Etotal error       : '+str(Etot_error)
print 'Total time         : '+str(total_time)


font = {'family':'serif','size':10}
plt.rc('font',**font)
fig = plt.figure(figsize=(3,2), dpi=80)
ax = fig.add_subplot(111)

N = mesh

asymp = N[-1]*Etot_error[-1]/N*1/1.1

ax.loglog(N, Etot_error, c='k', marker='o',ls=' ', mfc='w', ms=5, label='')
ax.loglog(N, asymp, c='k', marker='None', ls=':', lw=0.8, label=None)
ylim = [min(Etot_error)/1.2, max(Etot_error)*1.2]
ax.set_ylim(ylim)
ax.set_xlim(min(N)/1.2)

for axis in ['top','bottom','left','right']:
    ax.spines[axis].set_linewidth(0.5)

loc = (3*N[0]+N[1])/4
tex_loc_tot = array((loc,N[0]*Etot_error[0]/loc))
tex_angle_tot = math.atan2(log(abs(asymp[-1]-asymp[0])),log(abs(N[-1]-N[0])))*180/math.pi
ax.text(tex_loc_tot[0], tex_loc_tot[1]/1.6,r'N$^{-1}$',fontsize=8,rotation=-28,rotation_mode='anchor')
ax.set_ylabel('Relative error', fontsize=10)
ax.set_xlabel('Mesh density', fontsize=10)

fig.subplots_adjust(left=0.195, bottom=0.22, right=0.965, top=0.95)
fig.savefig('PyGBeOrientation_1IGTconvergence/1IGTconvergence.pdf',dpi=80,format='pdf')
